$debug

-- This table has the file transfer log of the form
-- the fields are [filename] [servfilename] [directorysaved]
cgilua.upload_transferlog = {}

-- Auxiliar C functions
cgilua.loadlibrary( 'upload', "uploadlib_open" )

--------------------------------------------
-- Renaming functions developed in C code
--------------------------------------------

cgilua.upl_read_net_input = cgilua_upl_read_net_input
cgilua_upl_read_net_input = nil
	
cgilua.upl_file_open = cgilua_upl_file_open
cgilua_upl_file_open = nil
	
cgilua.upl_file_close = cgilua_upl_file_close
cgilua_upl_file_close = nil
	
cgilua.upl_read_file_chunk = cgilua_upl_read_file_chunk
cgilua_upl_read_file_chunk = nil
	
cgilua.upl_write_file = cgilua_upl_write_file
cgilua_upl_write_file = nil

------------------------------------------------------------------------
--This function tests if it is a multipart/form-data enctype transfer.
------------------------------------------------------------------------
function cgilua.upl_is_multipart()

  local content_type = getenv("CONTENT_TYPE")
  local x, y, mpart = strfind(content_type, "(multipart/form%-data)")
  
  if mpart then
    return 1
  else
    return nil
  end
end

------------------------------------------------------------------------
-- This function calls a C coded function to read in binary mode the
-- standard input and save it in a temporary file on the server file system
------------------------------------------------------------------------
function cgilua.upl_read_stdin( default_root_dir )

  --lenght in bytes of the transmission buffer
  local size = tonumber( getenv( "CONTENT_LENGTH" ) )
  local path, temp_file = cgilua.splitpath(tmpname())
 
  temp_file = temp_file.."tmp"

  local ret = cgilua.upl_read_net_input( size, default_root_dir..temp_file )
  if ret then
    return 1, temp_file    
  else
    return nil, nil
  end
end

---------------------------------------------------------------------------
-- This function reads the temporary file for handling form fields
---------------------------------------------------------------------------
function cgilua.upl_handle_form( temp_file )

  --gets boundary value in CONTENT_TYPE enviroment variable
  local content_type = getenv("CONTENT_TYPE")
  x, y, boundary = strfind(content_type, "multipart/form%-data; boundary=(.-)$")
  
  --according to RFC1521, the prefix "--" should be added to the boundary.
  boundary = "--"..boundary 
  
  --call C Funtcion processfile
  local ret = cgilua_upl_process_file( temp_file, boundary )
  if not ret then
     error("Error processing temporary file.\n")
  end
end

------------------------------------------------------------------------------
-- This function process form field header
------------------------------------------------------------------------------
function cgilua.upl_process_header( header )

  local ACTION_SAVE = 1
  local ACTION_CGI  = 2
  local ACTION_NOTHING = 3

  -- Check if it is a file field
  local x, y, file_field = strfind(header, "(filename=)")

  if  file_field  then

    -- the file should only be saved if the filed is not blank
    local x, y, field_not_blank = strfind(header, "filename=\"(.-)\"")

    if  field_not_blank ~= ""  then

      -- get the file name being transfered
      local x, y, file_path = strfind(header, "filename=\"(.-)\"")            
      local path, fname = cgilua.splitpath(file_path)
       	
      -- get the rules file name                     
      local x, y, field_name = strfind(header, "[^e]name=\"(.-)\"")            
      local rules_file = cgi["rules_"..field_name]
            
      local name_to_save
      local dir_to_save
            
      if rules_file then
        -- there is a rules file for this field
        name_to_save, dir_to_save = cgilua.upl_getfilename(rules_file, fname)  
      else
        -- The default rules file should be used
        name_to_save, dir_to_save = cgilua.upl_getfilename(nil, fname)
      end
                
      if name_to_save then
     	cgilua.upl_addtransferlog(fname, name_to_save, dir_to_save)
   	return ACTION_SAVE, dir_to_save..name_to_save
      else
   	error("Error generating file name.\n")
      end                                       
    else
      return ACTION_NOTHING, ""
    end    
          
	else -- It is not a FILE field
	
	  local x, y, field_name = strfind(header, "name=\"(.-)\"")    
	       
	  if field_name  then
	    return ACTION_CGI, field_name
	  else
	    error("Bad field name. Cgi table entry was not added.\n")
	  end  	    
	
	end      
end

------------------------------------------------------------------
--This function check errors in the rules table
------------------------------------------------------------------
function cgilua.upl_checkrules( rules )
  if not rules then
    error("Table of rules not declared in the rules file.\n")
  end     
  if not rules.root_dir then
    error("Rules table does not contains [root_dir] required field.\n")
  end    
      
  -- Check if the directory path contains a slash as the end character
  local char = strsub(rules.root_dir, -1)
  if char ~= "\\" then
    rules.root_dir = rules.root_dir.."\\"
  end   
      
  return 1 
end

-------------------------------------------------------------------
-- This function prints transferlog table contents to 
-- filename or stdout
-------------------------------------------------------------------
function cgilua.upl_print_transferlog( filename )
  local oldout

  if filename then
    appendto(filename)
  else
    oldout = _STDOUT
    writeto()  
  end  
    
  local i = 1
  while cgilua.upload_transferlog[i] do
    write( "[upload time]: "..date( "%d/%m/%Y %H:%M:%S" ).. 
           "\t[original file]: "..cgilua.upload_transferlog[i].filename..
           "\t[renamed to]: "..cgilua.upload_transferlog[i].servname..
	   "\t[output dir]: "..cgilua.upload_transferlog[i].directory.."\n" ) 
    i = i + 1
  end

  -- stdout becomes the active output again
  if oldout then _STDOUT = oldout end
end

------------------------------------------------------------------
-- This functions adds an entry to the upload_transferlog table
------------------------------------------------------------------
function cgilua.upl_addtransferlog(filename, servname, directory )

  -- get the next free entry in the rules table
  local i = 1
  while cgilua.upload_transferlog[i] do i = i + 1 end
    
  cgilua.upload_transferlog[i] = {}
  cgilua.upload_transferlog[i].filename  = filename
  cgilua.upload_transferlog[i].servname  = servname
  cgilua.upload_transferlog[i].directory = directory        
end

------------------------------------------------------------------
-- This function returns the server new file name
------------------------------------------------------------------
function cgilua.upl_getfilename( rulesname, filename )
  local rules

  if not rulesname then
    rules = cgilua.uploadrules
  else
    rules = getglobal( rulesname )
  end

  cgilua.upl_checkrules( rules )

  local i = 1
  while rules[i] do
    local match = strfind(filename, rules[i].pattern)        
    if match then
      local name_to_save 
      if type(rules[i].func) == "function" then
        name_to_save = gsub(filename, rules[i].pattern, rules[i].func) 
      else
        name_to_save = gsub(filename, rules[i].pattern, rules[i].target)
      end
      return name_to_save, rules.root_dir   
    end
    i = i + 1
  end

  return filename, rules.root_dir
end

------------------------------------------------------------------
-- This function removes the temp files used by the script
------------------------------------------------------------------
function cgilua.upl_del_temp_files( temp_file )
  -- don't test return, windows systems sometime fails
  remove( temp_file )	
end

-------------------------------------------------------------------
-- This function proccess the hole form
-------------------------------------------------------------------
function cgilua.upl_form_upload()

  --local clk1 = clock()
    
  local default_root_dir
  -- check if root_dir has a slash in the end
  local char = strsub(cgilua.uploadrules.root_dir, -1)
  if char == "\\" then
    default_root_dir = cgilua.uploadrules.root_dir
  else
    default_root_dir = cgilua.uploadrules.root_dir.."\\"
  end   
    
  --local clk3 = clock()    
  local ret, temp_file = cgilua.upl_read_stdin( default_root_dir )
  --local clk4 = clock()
  if  ret  then
    cgilua.upl_handle_form( default_root_dir..temp_file )
    cgilua.upl_del_temp_files( default_root_dir..temp_file )
    cgilua.upl_print_transferlog()
  else
    error( "Some problems occurred on the data transmission to the server.\n"..
           "Check available server disk space or writing permissions.\n" )
  end
    
  --local clk2 = clock()
  --print( "tempo de execucao: "..tostring(clk2 - clk1) )
  --print( "tempo de temp: "..tostring(clk4 - clk3) )
end

  
cgilua.uploadlog = cgilua.upl_print_transferlog

